/*
 *  Copyright (c) 2011-2018, Zingaya, Inc. All rights reserved.
 */

#import <Foundation/Foundation.h>
#import "VIAudioDevice.h"

@protocol VIAudioManagerDelegate <NSObject>

/**
 * Event is triggered when active audio device or audio device that will be used for a further call is changed.
 *
 * @see <-[VIAudioManager currentAudioDevice]>
 *
 * @param audioDevice Activated audio device. See <VIAudioDevice>
 */
- (void)audioDeviceChanged:(VIAudioDevice *)audioDevice;

/**
 * Event is triggered when audio device can not be selected due to it is not available at this moment
 *
 * @see <-[VIAudioManager currentAudioDevice]>
 *
 * @param audioDevice Failed audio device. See <VIAudioDevice>
 */
- (void)audioDeviceUnavailable:(VIAudioDevice *)audioDevice;

/**
 * Event is triggered when new audio device connected / old audio device disconnected.
 *
 * @see <-[VIAudioManager availableAudioDevices]>
 *
 * @param availableAudioDevices List of newly available audio devices. See <VIAudioDevice>
 */
- (void)audioDevicesListChanged:(NSSet<VIAudioDevice *> *)availableAudioDevices;

@end

/** Interface that may be used to manage audio devices on iOS device. */
@interface VIAudioManager : NSObject

/**
 @warning NS_UNAVAILABLE
 */
- (instancetype)init NS_UNAVAILABLE;

/** Obtain VIAudioManager instance */
+ (instancetype)sharedAudioManager;

/**
 * Enable or disable loudspeaker
 *
 * @warning Deprecated. Use <-selectAudioDevice:>
 */
@property(nonatomic,assign) BOOL useLoudSpeaker
        __attribute__((deprecated("Use -selectAudioDevice:")));

/**
 * Delegate to monitor audio session route changes
 */
@property (nonatomic, weak) id<VIAudioManagerDelegate> delegate;

/**
 * Changes selection of the current active audio device.
 *
 * There are two cases:
 *
 * 1) before a call. The method doesn't activate an audio device, it just selects (i.e. points to) the audio device that will be activated
 *
 * 2) during a call. If the selected audio device is available, the method activates this audio device.
 *
 * Please note that active audio device can be later changed if new device is connected.
 * In this case <-[VIAudioManagerDelegate audioDeviceChanged:]> will be triggered to notify about new active device.
 *
 * If the application uses CallKit, you should take into consideration:
 *
 * * In case if Bluetooth headset is connected, audio routing depends on where a call is answered (from bluetooth headset or from phone screen).
 * Bluetooth headset will be activated only in case if a call is answered via Bluetooth headset controls. In other case the audio will be played via Receiver.
 *
 * * Audio is always routed to Bluetooth headset only in case if user selects "Bluetooth headset" as Call Audio Routing in the phone preferences.
 *
 * * In case if audio device is selected before CallKit activates the audio session, it is required to reselect this audio device after  `- (void)provider:(CXProvider *)provider didActivateAudioSession:(AVAudioSession *)audioSession` is called.
 * Otherwise audio routing may be reset to default.
 *
 * @param audioDevice Preferred audio device to use. See <VIAudioDevice>
 */
- (void)selectAudioDevice:(VIAudioDevice *)audioDevice;

/**
 * Returns active audio device during the call or audio device that will be used for a call if there is no calls at this moment.
 *
 * Please note that active audio device can be later changed if new device is connected.
 * In this case <-[VIAudioManagerDelegate audioDeviceChanged:]> will be triggered to notify about new active device.
 *
 * @return Currently active audio device. See <VIAudioDevice>
 */
- (VIAudioDevice *)currentAudioDevice;

/**
 * Returns the list of available audio devices.
 *
 * @return List of currently available audio devices. See <VIAudioDevice>
 */
- (NSSet<VIAudioDevice *> *)availableAudioDevices;

@end

@interface VIAudioManager (CallKit)
/**
 * Initializes AudioSession for use with CallKit integration.
 * Only needed for correct CallKit integration.
 *
 * Should be called in `- (void)provider:(CXProvider *)provider performStartCallAction:(CXStartCallAction *)action;`
 * and `- (void)provider:(CXProvider *)provider performAnswerCallAction:(CXAnswerCallAction *)action;` of CXProviderDelegate
 *
 * @see CXProviderDelegate
 *
 * @param error Occured error on configuration steps
 */
- (void)callKitConfigureAudioSession:(NSError **)error;

/**
 * Restores default AudioSession initialization routines, MUST be called if CallKit becomes disabled
 */
- (void)callKitReleaseAudioSession;

/**
 * Starts AudioSession
 * Only needed for correct CallKit integration.
 *
 * Should be called in `- (void)provider:(CXProvider *)provider didActivateAudioSession:(AVAudioSession *)audioSession;`
 * and `- (void)provider:(CXProvider *)provider performSetHeldCallAction:(CXSetHeldCallAction *)action;` of CXProviderDelegate
 *
 * @see CXProviderDelegate
 */
- (void)callKitStartAudio;

/**
 * Stops AudioSession
 * Only needed for correct CallKit integration.
 *
 * Should be called in `- (void)provider:(CXProvider *)provider performEndCallAction:(CXEndCallAction *)action;`
 * and `- (void)provider:(CXProvider *)provider performSetHeldCallAction:(CXSetHeldCallAction *)action;` of CXProviderDelegate
 *
 * @see CXProviderDelegate
 */
- (void)callKitStopAudio;

@end
